import('model.Component')
import('model.toolbar.content.sections.section.content.table.slots.Empty')
import('model.toolbar.content.sections.section.content.table.slots.Item')
import('model.toolbar.content.sections.section.content.table.slots.Tool')

---@class Row : Component
Row = Slot:extendAs("model.toolbar.content.sections.section.content.table.Row")

function Row:create(parent)
    return Component.create(self, parent, {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_content_sections_section_content_table_row" })
end

---@public
---@return Row
---@param root LuaGuiElement
function Row:new(parent, root)
    local instance = Row:super(Component:new(parent, root))
    instance:loadChildren()
    return instance
end

function Row:childrenClasses()
    return { Empty, Item, Tool }
end

function Row:isOccupied()
    return self:lastOccupiedSlotIndex() > 0
end

---@public
---@param width number
function Row:setWidth(width)
    self:trimTo(width)
    self:fill(width)
end

---@private
function Row:trimTo(width)
    for _, child in ipairs(self:children()) do
        if child:index() > width then
            child:delete()
        end
    end
end

---@public
---@return number
function Row:lastOccupiedSlotIndex()
    local lastOccupiedSlotIndex = 0
    for _, child in ipairs(self:children()) do
        if not child:isInstanceOf(Empty) then
            lastOccupiedSlotIndex = math.max(lastOccupiedSlotIndex, child:index())
        end
    end
    return lastOccupiedSlotIndex
end

---@private
---@param expectedSlots number
function Row:fill(expectedSlots)
    local missingCount = expectedSlots - self:slotsCount()
    assert(missingCount >= 0)
    local i = 0
    while i < missingCount do
        Empty:create(self)
        i = i + 1
    end
end

---@private
---@return number
function Row:slotsCount()
    return #self:slotsRoots()
end

---@private
---@return LuaGuiElement[]
function Row:slotsRoots()
    return self:root().children
end
